VERSION 1.0 CLASS
BEGIN
  MultiUse = 0   'False
END
Attribute VB_Name = "clsHelpAgent"
Attribute VB_Creatable = True
Attribute VB_Exposed = False
'How to use Help Agent
'Set HelpAgent = New clsHelpAgent
' Set help file name and owner window handle
'HelpAgent.HelpFileName = App.Path & "\" & "Project1.hlp"
'HelpAgent.hwnd = Me.hwnd
' Show the help file contents
'HelpAgent.ContextID = CLng(Val(Your topic#))
'HelpAgent.Action = HELP_CONTEXT <-- can use different help commands
Option Explicit

Dim ptHelpFileName          As String
Dim phWnd                   As Long
Dim plContextID             As Long
Dim ptKeyword               As String
Dim pnStatus                As Integer

Const HELP_CONTEXT = &H1          ' Display topic using context ID
Const HELP_QUIT = &H2             ' Terminate help
Const HELP_INDEX = &H3            ' Display the index index
Const HELP_CONTENTS = &H3&        ' Display the contents page
Const HELP_HELPONHELP = &H4       ' Display help on using help
Const HELP_SETINDEX = &H5         ' Set current index for multi index help
Const HELP_SETCONTENTS = &H5&     ' Set contents topic page
Const HELP_CONTEXTPOPUP = &H8&    ' Popup help window based on context ID
Const HELP_FORCEFILE = &H9&       ' Force a specific help file to be open
Const HELP_KEY = &H101            ' Display topic for given keyword
Const HELP_COMMAND = &H102&       ' Execute a help macro command
Const HELP_PARTIALKEY = &H105&    ' Display keyword help or search dialog
Const HELP_MULTIKEY = &H201&      ' Display topic using a multi keyword table
Const HELP_SETWINPOS = &H203&     ' Set the help window size and position

' Custom commands to position and size the help window
Const HELP_SHRINKTOPLEFT = &H1001&
Const HELP_SHRINKTOPRIGHT = &H1002&
Const HELP_SHRINKBOTTOMLEFT = &H1003&
Const HELP_SHRINKBOTTOMRIGHT = &H1004&

Private Type HELPWININFO
    wStructSize As Long
    x           As Long         ' Left
    y           As Long         ' Top
    dx          As Long         ' Width
    dy          As Long         ' Height
    wMax        As Long         ' Window style
    rgchMember  As String * 2
End Type

Const SW_SHOWNA = 8             ' Show normal window without focus

Private Declare Function WinHelp Lib "user32" Alias "WinHelpA" _
            (ByVal hwnd As Long, _
             ByVal lpHelpFile As String, _
             ByVal wCommand As Long, _
             ByVal dwData As Long) As Long

' Type safe declare for passing string commands to WinHelp
Private Declare Function WinHelpString Lib "user32" Alias "WinHelpA" _
            (ByVal hwnd As Long, _
             ByVal lpHelpFile As String, _
             ByVal wCommand As Long, _
             ByVal dwData As String) As Long

' Type safe declare for passing structures commands to WinHelp
Private Declare Function WinHelpStruct Lib "user32" Alias "WinHelpA" _
            (ByVal hwnd As Long, _
             ByVal lpHelpFile As String, _
             ByVal wCommand As Long, _
             dwData As Any) As Long


Property Let Action(lAction As Long)

Dim hwiHelpInfo         As HELPWININFO

' Check to be sure the help file has been specified
If Len(ptHelpFileName) = 0 Then
    ' No help file name available
    pnStatus = 1
    Exit Property
End If
    
' Set the data item and make the API call based
' on the requested action. The select case groups
' the actions by their common sets of API call
' parameter values.

Select Case lAction
    Case HELP_QUIT, HELP_INDEX, HELP_CONTENTS, HELP_HELPONHELP
        Call WinHelp(phWnd, ptHelpFileName, lAction, 0)
            
    Case HELP_KEY, HELP_PARTIALKEY
        If Len(ptKeyword) = 0 Then
            ' No index keyword specified
            pnStatus = 2
            Exit Property
        End If
        Call WinHelpString(phWnd, ptHelpFileName, lAction, ptKeyword)
            
    Case HELP_CONTEXT, HELP_CONTEXTPOPUP
        If Len(plContextID) = 0 Then
            ' No context ID specified
            pnStatus = 3
            Exit Property
        End If
        Call WinHelp(phWnd, ptHelpFileName, lAction, plContextID)
            
    Case HELP_SHRINKTOPLEFT, HELP_SHRINKTOPRIGHT, _
            HELP_SHRINKBOTTOMLEFT, HELP_SHRINKBOTTOMRIGHT
        hwiHelpInfo.wStructSize = Len(hwiHelpInfo)
        ' WinHelp always uses screen dimensions of 1024x1024
        ' regardless of the physical height and width, so the
        ' following value settings take that into account
        hwiHelpInfo.dx = 512
        hwiHelpInfo.dy = 512
        If lAction = HELP_SHRINKTOPLEFT Then
            hwiHelpInfo.x = 1
            hwiHelpInfo.y = 1
        End If
        If lAction = HELP_SHRINKTOPRIGHT Then
            hwiHelpInfo.x = 512
            hwiHelpInfo.y = 1
        End If
        If lAction = HELP_SHRINKBOTTOMLEFT Then
            hwiHelpInfo.x = 1
            hwiHelpInfo.y = 512
        End If
        If lAction = HELP_SHRINKBOTTOMRIGHT Then
            hwiHelpInfo.x = 512
            hwiHelpInfo.y = 512
        End If
        hwiHelpInfo.wMax = SW_SHOWNA
        Call WinHelpStruct(phWnd, ptHelpFileName, HELP_SETWINPOS, hwiHelpInfo)

End Select

End Property

Property Get HelpFileName() As String

' Return the help file name class property
HelpFileName = ptHelpFileName
    
End Property

Property Let hwnd(nHWnd As Long)

' Set the property that stores the hwnd of the
' parent window of the calling application
phWnd = nHWnd

End Property


Property Let HelpFileName(tHelpFileName As String)

' Set the help file name class property
ptHelpFileName = tHelpFileName

End Property


Property Get ContextID() As Long

' Return the context ID class property
ContextID = plContextID

End Property

Property Let ContextID(lContextID As Long)

' Set the context ID class property
plContextID = lContextID

End Property


Property Get Status() As Integer

' Return the status class property
Status = pnStatus
    
End Property

Property Get Keyword() As String

' Return the search/find keyword class property
Keyword = ptKeyword

End Property

Property Let Keyword(tKeyword As String)

' Set the search/find keyword class property
ptKeyword = tKeyword

End Property

